# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
# Date: 03.09.2016                                                            #
# Author: Ole-Johan Skrede                                                    #
#                                                                             #
# Solution proposal as part of the exercise program in                        #
# INF4300 - Digital image analysis at the University of Oslo                  #
#                                                                             #
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: #
"""
Solution proposal for exercise 1, task 2.
"""

# pylint: disable=expression-not-assigned
# pylint: disable=bad-indentation
# pylint: disable=redefined-outer-name

import os

import numpy as np
import cv2
import matplotlib.pyplot as plt

def plot_image(image, fig_num, name=None, colormap='gray', write_file=None):
  """Plot image"""
  fig = plt.figure(fig_num)
  plt.imshow(image, cmap=colormap, interpolation='none')
  if name is not None:
    plt.title(name)
  plt.xticks([]), plt.yticks([])
  plt.tight_layout()
  if write_file:
    fig.savefig(write_file, bbox_inches='tight', pad_inches=0)
  fig_num += 1
  return fig_num

def histogram(image):
  """Simple histogram function.

  Args:
    image: 2D numpy array of type uint8

  Returns:
    bins: 1D numpy array. Bin values used in the histogram.
    hist: 1D numpy array. Unnormalized histogram values.
  """
  assert len(image.shape) == 2, 'Grayscaled images only'
  assert image.dtype == 'uint8', '8bit unsigned int only'

  bin_size = 1
  bins = np.arange(0, 256, bin_size)

  hist = np.zeros(len(bins), dtype='int64')

  # Feel free to try different versions
  for k, _ in enumerate(bins):
    hist[k] = np.sum((image == bins[k])*1)

  # Check with numpy version
  np_hist, _ = np.histogram(image.flatten(), bins=256, range=[0, 255], density=False)

  for k, _ in enumerate(hist):
    assert hist[k] == np_hist[k], 'Numpy must be wrong in index ' + str(k)

  return bins, hist

def main():
  """Main"""

  print('='*80)
  print('Solution to weekly exercises in INF4300')
  print('Exercise 1')
  print('Task 2')
  print('-'*80)

  # Read 2D graylevel image
  img_filename = '../../images/coins.png' # Change this to suit you
  fig_num = 0
  write_dir = 'results/images'


  img = cv2.imread(img_filename, cv2.IMREAD_GRAYSCALE)
  fig_num = plot_image(img, fig_num,
                       write_file=os.path.join(write_dir, 'coins.png'))

  bins, hist = histogram(img)

  fig = plt.figure(1)
  plt.bar(bins, hist)
  fig.savefig(os.path.join(write_dir, 'coins_hist.png'), bbox_inches='tight',
              pad_inches=0)

  plt.show()

if __name__ == "__main__":
  main()
